#include <libxode.h>
#include <jabber/jabber.h>
#include <etherx.h>
#include <faim/aim.h>

#include "iq.h"

#ifndef INCL_AIMTRANS_H
#define INCL_AIMTRANS_H

#ifdef __cplusplus
extern "C" {
#endif

extern ehandler ehandler_at;
extern char *at_name;
extern int at_port;
extern ppdb PPDB;

typedef struct {
    pth_message_t head; /* the standard pth message header */
    jpacket p;
} _jpq, *jpq;

typedef struct at_login_struct {
    char *user;
    char *pass;
    jid from;
    jid to;
    jpacket jp;
} _at_login, *at_login;

typedef struct at_buddy_struct {
    pool p;
    jid from;
    jid to;
    /* Expand in here to cache info from the server */
	time_t t;
	struct at_buddy_struct *next;
} _at_buddy, *at_buddy;

/* Sessions */
typedef struct sessions_struct
{
    thread t;
    jid cur;
    jid from;
    struct aim_session_t ass;
    pool p;
    pth_msgport_t mp_to;
    int exit_flag;
    int loggedin;
    char *profile;
    char *screenname;
    char *password;
    at_buddy buddies;
    struct sessions_struct *next;
} _session, *session;

static char *msgerrreasons[] = {
  "Invalid error",
  "Invalid SNAC",
  "Rate to host",
  "Rate to client",
  "Not logged on",
  "Service unavailable",
  "Service not defined",
  "Obsolete SNAC",
  "Not supported by host",
  "Not supported by client",
  "Refused by client",
  "Reply too big",
  "Responses lost",
  "Request denied",
  "Busted SNAC payload",
  "Insufficient rights",
  "In local permit/deny",
  "Too evil (sender)",
  "Too evil (receiver)",
  "User temporarily unavailable",
  "No match",
  "List overflow",
  "Request ambiguous",
  "Queue full",
  "Not while on AOL"};
static int msgerrreasonslen = 25;

/* auth.c */
void at_sleeper_main(void *arg);
int at_auth_user(xmlnode xp);
void at_auth_subscribe(jid to, jid from, char *user, char *pass);

/* buddies.c */
void at_buddy_find_stale(time_t t);
at_buddy at_buddy_find(at_buddy head, jid j);
char *at_buddy_buildlist(session s, jid from);
int at_buddy_subscribe(jpacket jp);
int at_buddy_add(jpacket jp);
int at_parse_oncoming(struct aim_session_t *ass, 
                      struct command_rx_struct *command, ...);
int at_parse_offgoing(struct aim_session_t *ass, 
                      struct command_rx_struct *command, ...);
int at_parse_evilnotify(struct aim_session_t *sess, 
                              struct command_rx_struct *command, ...);
int at_parse_userinfo(struct aim_session_t *sess, 
                            struct command_rx_struct *command, ...);
int at_parse_locerr(struct aim_session_t *sess, 
                          struct command_rx_struct *command, ...);

/* external.c */
void at_init();
void at_etherx_receive(epacket e);

/* iq.c */
iqcb at_find_iqcb(const char *ns);
int at_register_iqns(const char *ns, iqfunc_t cb, void *arg);
int at_run_iqcb(const char *ns, jpacket jp);

/* messages.c */
int at_parse_incoming_im(struct aim_session_t *ass, 
                         struct command_rx_struct *command, ...);
int at_parse_misses(struct aim_session_t *sess, 
                    struct command_rx_struct *command, ...);
int at_parse_msgerr(struct aim_session_t *sess, 
                    struct command_rx_struct *command, ...);

/* parser.c */
int at_parse_packet(xmlnode x);

/* pres.c */
int at_server_pres(jpacket jp);
int at_session_pres(jpacket jp);

/* register.c */
int at_register(jpacket jp);

/* s10n.c */
int at_server_s10n(jpacket jp);
int at_session_s10n(jpacket jp);

/* sessions.c */
int at_parse_login(struct aim_session_t *sess, 
                   struct command_rx_struct *command, ...);
int at_parse_authresp(struct aim_session_t *sess, 
                      struct command_rx_struct *command, ...);
int at_serverready(struct aim_session_t *sess, 
                   struct command_rx_struct *command, ...);
int at_handleredirect(struct aim_session_t *sess,
                      struct command_rx_struct *command, ...);
int at_parse_ratechange(struct aim_session_t *sess, 
                              struct command_rx_struct *command, ...);
int at_parse_motd(struct aim_session_t *sess, 
                  struct command_rx_struct *command, ...);
int at_parse_connerr(struct aim_session_t *sess, 
                     struct command_rx_struct *command, ...);
int at_flapversion(struct aim_session_t *sess, 
                   struct command_rx_struct *command, ...);
int at_conncomplete(struct aim_session_t *sess, 
                    struct command_rx_struct *command, ...);
session at_session_create(char *user, char *pword, jid j, jid from);
void at_aim_session_parser(session s, jpacket jp);
void at_session_deliver(session s, xmlnode x, jid from);
void at_session_end(session s);
void at_session_unlink(session s);
void at_session_unlink_buddies(session s);
void at_session_buddy_unlink(session s, char *user);
session at_session_find_by_jid(jid j);
struct aim_conn_t *_aim_select(struct aim_session_t *ass, 
                               pth_event_t ev, 
                               int *status);

/* utils.c */
char *strip_html(char * text,pool p);
void at_psend(pth_msgport_t mp, jpacket p);
char *str_to_UTF8(pool p,unsigned char *in);
unsigned char *UTF8_to_str(pool p,unsigned char *in);
char *at_normalize(char *s);

/* unknown.c */
void *at_unknown_main(void *arg);

#ifdef __cplusplus
}
#endif

#endif /* INCL_AIMTRANS_H */
