#include "aimtrans.h"

at_buddy buddies=NULL;
pth_mutex_t budmutex = PTH_MUTEX_INIT;

void at_buddy_find_stale(time_t t)
{
	at_buddy cur;
	at_buddy prev;
	
	if(t == 0 || buddies == NULL)
		return;

    pth_mutex_acquire(&budmutex, 0, NULL);

	prev = NULL;
	cur = buddies;
	while(cur)
	{
		if(cur->t+60 < t)
		{
			if(prev == NULL)
			{
				prev = cur->next;
				pool_free(cur->p);
				buddies = cur = prev;
				prev = NULL;
			}
			else
			{
				prev->next=cur->next;
				pool_free(cur->p);
			}
		}
		else
		{
			cur = cur->next;
		}
	}

    pth_mutex_release(&budmutex);

	return;

}

at_buddy at_buddy_find(at_buddy head, jid j)
{
    at_buddy cur;

    if(head == NULL || j == NULL)
        return NULL;

    for(cur = head; cur != NULL; cur = cur->next)
    {
        if(cur->p)
            if(jid_cmp(cur->to, j)==0)
                return cur;
    }

    return NULL;
}

char *at_buddy_buildlist(session s, jid from)
{
	at_buddy next;
    at_buddy new;
	char *blist;
	spool sp;
	int found=0;
    pool p, bp;

    p = pool_new();
	sp = spool_new(p);

	at_buddy_find_stale(time(NULL));

	log_debug(ZONE, "[AIM] Getting buddies for %s\n", jid_full(from));
	if(from == NULL || buddies == NULL)
	{
		log_debug(ZONE, "[AIM] Something is null (probably buddies), exiting.");
		return NULL;
	}
    pth_mutex_acquire(&budmutex, 0, NULL);
	for(next = buddies;next != NULL; next = next->next)
	{
		/* Check for borked data */
		if(next == NULL || next->from == NULL)
			break;

        log_debug(ZONE, "Comparing %s and %s", jid_full(next->from), jid_full(from));
		if(j_strcmp(jid_full(next->from),jid_full(from))==0)
		{
			log_debug(ZONE, "[AIM] Adding a buddy %s\n", jid_full(next->to));
			
			spooler(sp, next->to->user, "&", sp);
            bp = pool_new();
            new = pmalloco(bp, sizeof(_at_buddy));
            new->p = bp;
            new->to = jid_new(bp, jid_full(next->to));
            new->from = NULL;
            new->t = 0;
            new->next = s->buddies;
            s->buddies = new;

			found = 1;
			next->t = time(NULL)+31;
		}
	}

	if(found)
	{
		blist = strdup(spool_print(sp));
	}
	else
	{
		blist = NULL;
	}
	
    pool_free(p);
    pth_mutex_release(&budmutex);
	return blist;
}

int at_buddy_subscribe(jpacket jp)
{
    xmlnode dup,dup2;
    jid from,to;
    session s=at_session_find_by_jid(jp->from);

    /* check for a valid session */
    if(s==NULL)
    {
        xmlnode err,error;
        /* if no session, and subscribing to a buddy, not aim-t */
        err=xmlnode_new_tag("message");
        xmlnode_put_attrib(err,"type","error");
        xmlnode_put_attrib(err,"from",at_name);
        xmlnode_put_attrib(err,"to",jid_full(jp->from));
        error=xmlnode_insert_tag(err,"error");
        xmlnode_insert_cdata(error,"Cannot Subscribe to a AIM Buddy without a registration",-1);
        xmlnode_put_attrib(error,"code","407");
        ehandler_send(ehandler_at,err,jp->from->server);

        return 0;
    }
    /* XXX
	 * Create the duplicate for the subscribe notice
	 * Construct a subscribed notification
	 * 
	 * I had to do these in this odd order due to memory concerns
	 * Dunno why I didn't just make two dups.
	 *
	 */
	dup = xmlnode_dup(jp->x);
	dup2 = xmlnode_dup(jp->x);

	xmlnode_put_attrib(dup, "to", jid_full(jp->from));
	xmlnode_put_attrib(dup, "from", jid_full(jp->to));
	xmlnode_put_attrib(dup, "type", "subscribed");
	log_debug(ZONE, "[AIM] Sending subscribed notice\n");
	ehandler_send(ehandler_at, dup, jp->from->server);

	/* Construct the subscribe back */
	xmlnode_put_attrib(dup2, "type", "subscribe");
    xmlnode_put_attrib(dup2, "to", jid_full(jp->from));
    xmlnode_put_attrib(dup2, "from", jid_full(jp->to));
	log_debug(ZONE, "[AIM] Asking for a subscribe\n");
	ehandler_send(ehandler_at, dup2, jp->from->server);

    /* XXX do we free the jp? */
	return 0;
}

int at_buddy_add(jpacket jp)
{
	session s;
	at_buddy buddy;
    pool p;
	int newbud=0;
	
    log_debug(ZONE, "[AIM] Going to add a buddy\n");
	s = at_session_find_by_jid(jp->from);
	if(s==NULL) {
		newbud = 1;
	}
	else {
		if(s->loggedin == 0)
			newbud=1;
		else
			newbud=0;
	}
		
	if(newbud)
	{
        pth_mutex_acquire(&budmutex, 0, NULL);

        if(at_buddy_find(buddies, jp->to)!=NULL)
            return 0;
		log_debug(ZONE, "[AIM] New buddy in queue: %s\n",jid_full(jp->to));
		at_buddy_find_stale(time(NULL));
        p = pool_new();
		buddy = pmalloco(p, sizeof(_at_buddy));
        buddy->p = p;
		buddy->to=jid_new(p, jid_full(jp->to));
		buddy->from=jid_new(p, jid_full(jp->from));
		buddy->t=time(NULL);
		buddy->next=buddies;
		buddies=buddy;
        pth_mutex_release(&budmutex);
		return 0;
	}
	else
	{
        pth_mutex_acquire(&budmutex, 0, NULL);
        if(at_buddy_find(s->buddies, jp->to)==NULL)
        {
    		log_debug(ZONE, "[AIM] Adding buddy %s to %s\n", jp->to->user, jid_full(jp->from));
            p = pool_new();
            buddy = pmalloco(p, sizeof(_at_buddy));
            buddy->p = p;
            buddy->to=jid_new(p, jid_full(jp->to));
            buddy->from = jid_new(p, jid_full(jp->from));
            buddy->t=0;
            buddy->next = s->buddies;
            s->buddies=buddy;
            at_buddy_subscribe(jp);
        }
		aim_add_buddy(&s->ass, aim_getconn_type(&s->ass, AIM_CONN_TYPE_BOS),
                       jp->to->user);
        pth_mutex_release(&budmutex);
	}
    
    /* Return type?  Free the jp? */
    return 0;
}

int at_parse_oncoming(struct aim_session_t *ass, 
                      struct command_rx_struct *command, ...)
{

    xmlnode away;
    xmlnode status;
	xmlnode x;
	session s;
    jpacket jp;
    char *msg;
    struct aim_userinfo_s *userinfo;

    va_list ap;
    va_start(ap, command);
    userinfo = va_arg(ap, struct aim_userinfo_s *);
    va_end(ap);

    log_debug(ZONE, "Oncoming buddy %s", userinfo->sn);

	s = (session)ass->aux_data;

	x = xmlnode_new_tag("presence");
	xmlnode_put_attrib(x, "to", jid_full(s->cur));
	xmlnode_put_attrib(x, "from", at_name);
	jp = jpacket_new(x);
	jid_set(jp->from, at_normalize(userinfo->sn), JID_USER);
	xmlnode_put_attrib(jp->x, "from", jid_full(jp->from));
    if(userinfo->flags&AIM_FLAG_AWAY)
    {
        away = xmlnode_insert_tag(x, "show");
        xmlnode_insert_cdata(away, "away", 4);
    }
    status = xmlnode_insert_tag(x, "status");
    msg = pmalloco(xmlnode_pool(x), 24);
    memset(msg, '\0', 24);
    sprintf(msg, "Idle %d Minutes", (userinfo->idletime));
    xmlnode_insert_cdata(status, msg, -1);
	ehandler_send(ehandler_at, jp->x, s->cur->server);

	return 1;
}

int at_parse_offgoing(struct aim_session_t *ass, 
                      struct command_rx_struct *command, ...)
{

	xmlnode x;
	jpacket jp;
	session s;
    at_buddy cur, prev;
    
	s = (session)ass->aux_data;

	x = xmlnode_new_tag("presence");
	xmlnode_put_attrib(x, "to", jid_full(s->cur));
	xmlnode_put_attrib(x, "from", at_name);
	xmlnode_put_attrib(x, "type","unavailable");
	jp = jpacket_new(x);
	jid_set(jp->from, at_normalize(&(command->data[11])), JID_USER);
	xmlnode_put_attrib(jp->x, "from", jid_full(jp->from));

    at_session_buddy_unlink(s, jp->from->user);
	ehandler_send(ehandler_at, jp->x, s->cur->server);

	return 1;
}

int at_parse_evilnotify(struct aim_session_t *sess, 
                              struct command_rx_struct *command, ...)
{
    va_list ap;
    char *sn;
    xmlnode x;
    xmlnode body;
    jpacket jp;
    session s;
    char msg[100];

    va_start(ap, command);
    sn = va_arg(ap, char *);
    va_end(ap);

    memset(&msg, '\0', 100);
    snprintf((char *)&msg, 100, "Warning from: %s", sn);

    s = (session)sess->aux_data;

	x = xmlnode_new_tag("message");
	xmlnode_put_attrib(x, "to", jid_full(s->cur));
	xmlnode_put_attrib(x, "from", at_name);
	xmlnode_put_attrib(x, "type","error");
	body = xmlnode_insert_tag(x, "error");
    xmlnode_insert_cdata(body, (char *)&msg, strlen((char *)&msg));

    jp = jpacket_new(x);
	ehandler_send(ehandler_at, jp->x, s->cur->server);

    return 1;
}

int at_parse_userinfo(struct aim_session_t *sess, 
                            struct command_rx_struct *command, ...)
{
    /* XXX Do something here? */
    return 1;
}


int at_parse_locerr(struct aim_session_t *sess, 
                          struct command_rx_struct *command, ...)
{
/* XXX Do we want this?
    va_list ap;
    char *destsn;
    unsigned short reason;
    char *sn;
    xmlnode x;
    xmlnode body;
    jpacket jp;
    session s;
    char msg[1024];

    va_start(ap, command);
    destsn = va_arg(ap, char *);
    reason = va_arg(ap, unsigned short);
    va_end(ap);

    memset(&msg, '\0', 1024);
    snprintf("User information for %s unavailable (reason 0x%04x: %s)\n", destsn, reason, (reason<msgerrreasonslen)?msgerrreasons[reason]:"unknown");

    s = (session)ass->aux_data;

	x = xmlnode_new_tag("message");
	xmlnode_put_attrib(x, "to", jid_full(s->j));
	xmlnode_put_attrib(x, "from", at_name);
	xmlnode_put_attrib(x, "type","error");
	body = xmlnode_insert_tag(x, "error");
    xmlnode_insert_cdata(body, &msg, strlen(&msg));

    jp = jpacket_new(x);
	ehandler_send(ehandler_at, jp->x, s->j->server);
*/  
    return 1;
}
