#include "aimtrans.h"

int at_server_pres(jpacket jp)
{
    session s;
    xmlnode x;

    switch(jpacket_subtype(jp))
    {
        case JPACKET__AVAILABLE:
            if(jp->to->resource == NULL)
			{
                return at_buddy_add(jp);
            }
            s=at_session_find_by_jid(jp->from);
            if(s)
                return at_session_pres(jp);
            PPDB=ppdb_insert(PPDB,jp->from,jp->x);
            log_debug(ZONE, "[AIM] Starting a new session!\n");
            return at_auth_user(xmlnode_dup(jp->x));
        case JPACKET__UNAVAILABLE:
            /* XXX These two depend on sessions so we'll do those first  */
            /* XXX If we get avail pres to a user and we have no session 
             *     then we need to put it into a list for when the session
             *     becomes active.  Client side buddy lists are such a flaw
             *     in good IM design.
             */
        case JPACKET__PROBE:
            /* XXX Still not sure if we want to handle this.
             *     What are the pros and cons of doing this?
             */
        default:
            jutil_error(jp->x, TERROR_NOTACCEPTABLE);
            ehandler_send(ehandler_at, jp->x, jp->from->server);
            return 1;
    }

    return 0;
}

int at_session_pres(jpacket jp)
{
    xmlnode x;
    session s;
    jid j;

    s = at_session_find_by_jid(jp->from);

    switch(jpacket_subtype(jp))
    {
        case JPACKET__PROBE:
            log_debug(ZONE, "[AIM] Probed, no logical way to handle\n");
            return 0;
        case JPACKET__UNAVAILABLE:
            /* insert into ppdb, check to see if we should go offline */
            PPDB=ppdb_insert(PPDB,jp->from,jp->x);
            j=jid_new(jp->p,jid_full(jp->from));
            jid_set(j,NULL,JID_RESOURCE);
            x=ppdb_primary(PPDB,j);
            if(x!=NULL)
            {
                /* not last resource, don't go offline yet */
                log_debug(ZONE,"[AIM] active resources, not ending session");
                s->cur = jid_new(s->p,xmlnode_get_attrib(x, "from"));
                return 0;
            }
			log_debug(ZONE, "[AIM] Telling the session to end!\n");
            if(s==NULL)
                return 0;
			at_session_end(s);
            return 0;
		case JPACKET__AVAILABLE:
                PPDB=ppdb_insert(PPDB,jp->from,jp->x);
				log_debug(ZONE, "[AIM] Starting a new session!\n");
                s=at_session_find_by_jid(jp->from);
                log_debug(ZONE, "[AIM] S is: %X", s);
				if((s) && (xmlnode_get_tag(jp->x, "show") != NULL))
                {
                    /* why are we returning here? 
                    * this doesn't seem like the proper way
                    * to handle this...
                    */   
                    char *awaymsg;
                    char profile[]="";

                    awaymsg = xmlnode_get_tag_data(jp->x, "status");
                    log_debug(ZONE, "[AIM] Setting user away\n");
                    aim_bos_setprofile(&s->ass, 
                            aim_getconn_type(&s->ass, AIM_CONN_TYPE_BOS),
                            profile, awaymsg, AIM_CAPS_CHAT);
                    /* at_psend(s->mp_to, jp, 0); */
                    x = xmlnode_dup(jp->x);
                    xmlnode_put_attrib(x, "from", jid_full(s->from));
                    xmlnode_put_attrib(x, "to", jid_full(s->cur));
                    log_debug(ZONE, "[AIM] Pres Send: %s", xmlnode2str(x));
                    ehandler_send(ehandler_at, x, s->cur->server);
					return 0;
                }
				if(jp->to->resource == NULL)
				{
					return at_buddy_add(jp);
				} else {
                    char profile[] = "";
                    /* user is already connected, don't start a new session */
                    /* No show let's reset their profile */
                    log_debug(ZONE, "[AIM] Setting user away\n");
                    if(s!=NULL)
                    {
                        aim_bos_setprofile(&s->ass, 
                                aim_getconn_type(&s->ass, AIM_CONN_TYPE_BOS),
                                profile, NULL, AIM_CAPS_CHAT);
                        x = xmlnode_dup(jp->x);
                        xmlnode_put_attrib(x, "from", jid_full(s->from));
                        xmlnode_put_attrib(x, "to", jid_full(s->cur));
                        log_debug(ZONE, "[AIM] Pres Send: %s", xmlnode2str(x));
                        ehandler_send(ehandler_at, x, s->cur->server);
                        return 0;
                    }
					return at_auth_user(xmlnode_dup(jp->x));
				}
                x = xmlnode_dup(jp->x);
				xmlnode_put_attrib(x, "from", jid_full(jp->to));
				xmlnode_put_attrib(x, "to", jid_full(jp->from));
				ehandler_send(ehandler_at, x, jp->from->server);

                return 0;
    }

    return 0;
}
